/** 
 *  Hyper Operating System V4 Advance
 *
 * @file  act_tsk.c
 * @brief %jp{^XN̋N}%en{Activate Task}
 *
 * Copyright (C) 1998-2006 by Project HOS
 * http://sourceforge.jp/projects/hos/
 */



#include "core/core.h"



#if _KERNEL_SPT_ACT_TSK


/** %jp{^XN̋N}%en{Activate Task}
 * @param  tskid    %jp{NΏۂ̃^XNIDԍ}%en{ID number of the task to be activated}
 * @retval E_OK     %jp{I}%en{Normal completion}
 * @retval E_ID     %jp{sIDԍ(tskids邢͎gpłȂ)}%en{Invalid ID number(tskid is invalid or unusable)}
 * @retval E_NOEXS  %jp{IuWFNg(Ώۃ^XNo^)}%en{Non-existant object(specified task is not registerd)}
 * @retval E_QOVR   %jp{L[COI[o[t[(NvL[CÕI[o[t[)}%en{Queue overflow(overflow of activate queuing count)}
 */
ER act_tsk(
		ID tskid)
{
	_KERNEL_T_TSKHDL		tskhdl;
	_KERNEL_T_TCB_PTR		tcb;
	_KERNEL_T_TCB_RO_PTR	tcb_ro;

	if ( tskid == TSK_SELF )		/* %jp{^XNw莞̕ϊ} */
	{
#if _KERNEL_SPT_ACT_TSK_E_ID
		if ( _KERNEL_SYS_SNS_CTX() )
		{
			return E_ID;		/* %jp{sIDԍ} */
		}
#endif	
		/* %jp{s^XN擾} */
		tskhdl = _KERNEL_SYS_GET_RUNTSK();

		_KERNEL_ENTER_SVC();		/* %jp{enter service-call}%jp{T[rXR[ɓ} */
	}
	else
	{
#if _KERNEL_SPT_ACT_TSK_E_ID
		if ( !_KERNEL_TSK_CHECK_TSKID(tskid) )
		{
			return E_ID;	/* %jp{sIDԍ} */
		}
#endif
		
		_KERNEL_ENTER_SVC();		/* %jp{enter service-call}%jp{T[rXR[ɓ} */
		
		/* %jp{IuWFNg݃`FbN} */
#if _KERNEL_SPT_ACT_TSK_E_NOEXS
		if ( !_KERNEL_TSK_CHECK_EXS(tskid) )
		{
			_KERNEL_LEAVE_SVC();		/* %jp{leave service-call}%jp{T[rXR[o} */
			return E_NOEXS;			/* %jp{IuWFNg} */
		}
#endif
		/* IDԍw莞̕ϊ */
		tskhdl = _KERNEL_TSK_ID2TSKHDL(tskid);
	}
	

	/* %jp{TCB擾} */
	tcb = _KERNEL_TSK_TSKHDL2TCB(tskhdl);

	if ( _KERNEL_TSK_GET_TSKSTAT(tcb) != _KERNEL_TTS_DMT )		/* %jp{IuWFNgԔ} */
	{
#if _KERNEL_TCB_ACTCNT
		{
			_KERNEL_TSK_T_ACTCNT actcnt;

			/* %jp{ɎsԂȂNvL[CO} */
			actcnt = _KERNEL_TSK_GET_ACTCNT(tcb);
		
			/* %jp{ɎsԂȂNvL[CO} */
		#ifdef _KERNEL_CFG_ACT_TSK_E_QOVR
			if ( actcnt >= _KERNEL_TMAX_ACTCNT )
			{
				_KERNEL_LEAVE_SVC();
				return E_QOVR;		/* %jp{L[COI[o[t[} */
			}
		#endif

			/* %jp{NvL[CO} */
			_KERNEL_TSK_SET_ACTCNT(tcb, actcnt + 1);
		}
#else
		{
			_KERNEL_LEAVE_SVC();
			return E_QOVR;		/* %jp{L[COI[o[t[} */
		}
#endif
	}
	else
	{
		/* %jp{^XNԏ} */
		tcb_ro = _KERNEL_TSK_GET_TCB_RO(tskid, tcb);
		_KERNEL_TSK_SET_TSKPRI(tcb, _KERNEL_TSK_GET_ITSKPRI(tcb_ro));
		_KERNEL_TSK_SET_TSKBPRI(tcb, _KERNEL_TSK_GET_ITSKPRI(tcb_ro));
		_KERNEL_TSK_SET_TSKSTAT(tcb, _KERNEL_TTS_RDY);
		_KERNEL_TSK_SET_ACTCNT(tcb, 0);
		_KERNEL_TSK_SET_WUPCNT(tcb, 0);
		_KERNEL_TSK_SET_SUSCNT(tcb, 0);
		
		/* %jp{ReLXg} */
		_KERNEL_CRE_CTX(
				_KERNEL_TSK_GET_CTXCB(tcb),					/* %jp{ReLXgubN} */
				_KERNEL_TSK_GET_STKSZ(tcb_ro),				/* %jp{^XÑX^bN̈TCY} */
				_KERNEL_TSK_GET_STK(tcb_ro),				/* %jp{^XÑX^bN̈̐擪Ԓn} */
				(VP)_KERNEL_TSK_GET_ISP(tcb_ro),			/* %jp{X^bN|C^̏l} */
				(FP)_kernel_ent_tsk,						/* %jp{ReLXg̊JnAhX} */
				(VP_INT)_KERNEL_TSK_GET_EXINF(tcb_ro),		/* %jp{^XN̊g} */
				(VP_INT)_KERNEL_TSK_GET_TASK(tcb_ro)		/* %jp{^XN̋NԒn} */
			);
		
		/* %jp{^XNs\Ԃɐݒ} */
		_KERNEL_DSP_STA_TSK(tskhdl);
		
		/* %jp{^XNfBXpb`̎s}%en{task dispatch} */
		_KERNEL_DSP_TSK();
	}
	
	_KERNEL_LEAVE_SVC();
	
	return E_OK;	/*  */
}


#else	/* _KERNEL_SPT_ACT_TSK */


/** %en{Activate Task}%jp{^XN̋N}
 * @param  tskid    %jp{NΏۂ̃^XNIDԍ}%en{ID number of the task to be activated}
 * @retval E_NOSPT  %jp{T|[g@\}%en{Unsupported function}
 */
ER act_tsk(
		ID tskid)
{
	return E_NOSPT;
}

#endif	/* _KERNEL_SPT_ACT_TSK */


/* end of file */
